<?php

$ConfigFileName = dirname(__FILE__).'/../Config/Config.php';
if(file_exists($ConfigFileName)) include_once($ConfigFileName);

include_once(dirname(__FILE__).'/../Global.php');
include_once(dirname(__FILE__).'/Version.php');
include_once(dirname(__FILE__).'/View.php');
include_once(dirname(__FILE__).'/UpdateTrace.php');
include_once(dirname(__FILE__).'/DefaultConfig.php');

class Core extends Application
{
  var $Pages;
  var $ShowPage;
  var $BaseURL;
  var $UseSession;

  function __construct()
  {
    parent::__construct();
    $this->Pages = array();
    $this->UseSession = true;
    $this->ShowPage = true;
    $this->BaseURL = $_SERVER['SCRIPT_NAME'];
    if(substr($this->BaseURL, -10, 10) == '/index.php')
      $this->BaseURL = substr($this->BaseURL, 0, -10);
  }

  function RunCommon()
  {
    global $Config, $DatabaseRevision, $WithoutSessionStart;

    if($this->UseSession) session_start();

    $ErrorHandler = new ErrorHandler();
    $ErrorHandler->ShowError = $Config['Web']['ShowError'];
    //$ErrorHandler->Start();

    try {
      $this->Database = new Database();
      $this->Database->Connect($Config['Database']['Host'], $Config['Database']['User'],
        $Config['Database']['Password'], $Config['Database']['Database']);
      $this->Database->Prefix = $Config['Database']['Prefix'];
      $this->Database->charset($Config['Database']['Charset']);
      $this->Database->ShowSQLError = $Config['Web']['ShowSQLError'];
      $this->Database->ShowSQLQuery = $Config['Web']['ShowSQLQuery'];
    } catch (Exception $E) {
    }

    // SQL injection hack protection
    foreach($_POST as $Index => $Item) $_POST[$Index] = addslashes($Item);
    foreach($_GET as $Index => $Item) $_GET[$Index] = addslashes($Item);

    $this->Config = &$Config;

    // Register and start existing modules
    $this->Setup = new Setup($this);
    $this->Setup->Start();
    if($this->Setup->CheckState())
    {
      $this->ModuleManager->Start();
    }
  }

  function Run()
  {
    $this->RunCommon();
    if($this->ShowPage)
    {
      $this->PathItems = ProcessURL();
      $this->ShowPage();
    }
  }

  function ShowPage()
  {
    $this->BaseView = new BaseView($this);

    /* @var $Page Page */
    $ClassName = $this->SearchPage($this->PathItems, $this->Pages);
    if($ClassName != '')
    {
      $Page = new $ClassName($this);
    } else {
      $Page = new PageMissing($this);
    }
    echo($this->BaseView->GetOutput($Page));
  }

  function RegisterPage($Path, $Handler)
  {
    if(is_array($Path))
    {
      $Page = &$this->Pages;
      $LastKey = array_pop($Path);
      foreach($Path as $PathItem)
      {
        $Page = &$Page[$PathItem];
      }
      if(!is_array($Page)) $Page = array('' => $Page);
      $Page[$LastKey] = $Handler;
    } else $this->Pages[$Path] = $Handler;
  }

  function UnregisterPage($Path)
  {
    unset($this->Pages[$Path]);
  }

  function SearchPage($PathItems, $Pages)
  {
    if(count($PathItems) > 0) $PathItem = $PathItems[0];
      else $PathItem = '';
    if(array_key_exists($PathItem, $Pages))
    {
      if(is_array($Pages[$PathItem]))
      {
        array_shift($PathItems);
        return($this->SearchPage($PathItems, $Pages[$PathItem]));
      } else return($Pages[$PathItem]);
    } else return('');
  }

  function Link($Target)
  {
    return($this->BaseURL.$Target);
  }
}

class PageMissing extends Page
{
  function __construct($System)
  {
    parent::__construct($System);
    $this->ParentClass = '';
    $this->Title = T('Page not found');
  }

  function Show()
  {
    Header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found');
    return('<h3 align="center">'.T('Required page not found'));
  }
}
